<?php
/* $Id: dbfm_file.inc,v 1.1.2.19 2008/11/19 13:29:48 geoffeagles Exp $ */

/**
 * dbfm_mkdir -called from the ajax action - switch case 'mkdir':
 * @param rec $parentrec - a pointer to the parent record
 * @param string $dest - it's the proposed new directory name
 * @param bool $rename - This flag allows a different directory name to be chosen for cases where the directory already exists
 *                       if you were trying to create a directory myfiles then it would try to rename it 
 *                       myfiles_01, myfiles_02 etc. until it found a name that hadn't been used
 * @param string $error - the address of an array used to store any error messages
 * @ret   bool   - true if the directory is created successfully
 */

function dbfm_mkdir($parentrec, $dest, $rename = TRUE, &$error) {
 global $user;

 $currentdate = date("Y-m-d H:i:s");
 $setmime = "directory";

 //if the parent is a version rather than a directory then you can't make a new directory
 if ($parentrec->fmime == 'version') {
   $error = "You cannot add directories within a version directory";
   return FALSE;
 }

 //we need to check first whether there's already a directory of this name, if there is then we
 // simply append a number and try again ... and again, eventually we'll find an unused name
 $count = 0;
 $ppath = $parentrec->fpath . '/' . $dest;   //proposed path
 $ptest = $ppath;
 $numret = 1;
 while ($numret) {
   $anythere = db_query("SELECT fid FROM {dbfm_file} WHERE fpath = '%s' AND fparent > -1", $ptest);
   $numret = db_num_rows($anythere);
   if ($numret) {
     $count++;
     $ptest = $ppath . "_" . $count;
   }
 }
 if ($count > 0) {
   $dest = $dest . "_" . $count;
 }
 //at this point $ptest should contain the new proposed path and $dest the new proposed directory name

 //increment fsize on the parent directory - shows the number of entries in the directory ZZZZ in the near future we may have a seperate
 //count of files and directories to update too
 $numdir = $parentrec->fsize + 1;

 /* ALREADY DONE _ AJAX PERMCHECK need to test whether we have permission to do this
 if (dbfm_check_perm($parentrec->fid, $accesstype = READ_WRITE) == FALSE) {
   $error[] = "You do not have the appropriate permission to add a directory here";
   return FALSE;
 */

 //claim a new file id (in a replicated system this is claimed from the master server)
 $newfid = db_next_id('{dbfm_file}_fid');  //doesn't return any sorta error.  I'm not sure about the name syntax

 // we'll write to the database (now includes the permissions)
 $result = db_query("INSERT INTO {dbfm_file} (fid, uid, fpath, fname, fsize, fmime, fcreatedate, flastmod, fparent, fraccess, fwaccess) VALUES (%d, %d, '%s', '%s', %d, '%s', '%s', '%s', %d, %d, %d)",$newfid, $user->uid, $ptest, $dest, 0, $setmime, $currentdate, $currentdate, $parentrec->fid, $parentrec->fraccess, $parentrec->fwaccess);
 if ($result === FALSE) {
   $error = "ERROR: Failed to insert the directory " . $dest . " into the database. fid = " . $newfid;
   return FALSE;
 }

 // and update the parent record - fsize stores the number of directories parented, date must be updated for correct directory refreshes
 $result = db_query("UPDATE {dbfm_file} SET `fsize` = %d, `flastmod` = '%s' WHERE fid = %d ", $numdir, $currentdate, $parentrec->fid);
 if ($result === FALSE) {
   $error = "ERROR: Failed to update the parent directory for " . $dest;
   return FALSE;
 }

 //now we need to set permissions for this new directory - start by reading the parent directories permissions (if there are any)
 $qperm = "SELECT * FROM {dbfm_perm} WHERE fid = " . $parentrec->fid;
 $pperms = db_query($qperm);
 if ($pperms == FALSE) {
   $error = "Error reading the default permissions";
   return FALSE;
 }
 $numret = db_num_rows($pperms);

 //if there are any records to copy (the initial default is to have no permissions)
 if ($numret > 0) {
   //make copies of each parent permission record in turn
   while ($perm_rec = db_fetch_object($pperms)) {
     insert_perms($newfid, $perm_rec->rgaccess, $perm_rec->rgid);
   }
 }

 //return the directory path of the newly created directory in the error array 
 //SUCCESS!!
 $error = $ptest;
 return TRUE;
}

/**
 * dbfm_rename - called from the ajax action - switch case 'rename' 
 *
 * @param record  $source - db file record for the file or directory to be modified
 * @param string  $new - the proposed new name
 * @param string &$err - ref to error array for client feedback
 *
 * @ret   bool   -true if the directory /version/ file is renamed and file record(s) updated
 *
 * note that the permissions check has already been carried out by the calling javascript as has the check on whether the file / version / 
 * directory name is valid
 */
function dbfm_rename($source, $new, &$err) {
  $currentdate = date("Y-m-d H:i:s");

  //are source and dest the same - if so then exit doing nothing
  if ($source->fname == $new) {
    $err = 'Name has not changed';
    return FALSE;
  } 
  
  //checks on whether extensions are valid, name lengths are valid, etc. etc.  are done elsewhere
  
  //does the destination already exist - this has to be done with a path 
  //strip back to the last slash then append the new filename - the calling code has vetted the new filename
  $filepath = strrev(substr(strrev($source->fpath), strpos(strrev($source->fpath),'/')));
  $dest = $filepath . $new;
  $qdest = db_query("SELECT fid FROM {dbfm_file} WHERE fpath = '%s' AND fparent > -1", $dest);
  $numret = db_num_rows($qdest);
  if ($numret > 0) {
    $err = "Rename failed - destination " . $dest . " already exists";
    return FALSE;
  }

  //if there's already a title that's NOT the original filename then leave it - otherwise overwrite with the new filename
  if ($source->fname == $source->ftitle) {
    $newtitle = $new;
  }
  else {
   $newtitle = $source->ftitle;
  }
  
  $result = db_query("UPDATE {dbfm_file} SET fpath = '%s', fname = '%s', ftitle = '%s', flastmod = '%s' WHERE fid = %d", $dest, $new, $newtitle, $currentdate, $source->fid);
  if($result === FALSE) {
    $err = "ERROR: Rename failed. Couldn't change name to " . $new;
    return FALSE;
  }
  
  //we also need to modify the lastmodified date for the parent directory to ensure that the browser can automatically refresh
  $result = db_query("UPDATE {dbfm_file} SET flastmod = '%s' WHERE fid = %d", $currendate, $source->fparent);
  

  //if we rename a directory or version there may be files, directories and versions underneath it
  if (($source->fmime == 'directory' || $source->fmime == 'version') && $source->fsize > 0) {
    //if it's a directory we need to recurse through the database changing the path for all the children
    //need to update the sourec record first
    $source->fpath = $dest;
    $ret = TRUE;
    return dbfm_recur_dbrename($source, $ret , $err);
  }
  return TRUE;
  
}

/**
 * dbfm_recur_dbrename -the function to recursively rename (actually repath) database records
 *
 * @param record  $source - the source directory record
 * @param bool  $ret - the return value from the recursive function
 * @param string &$err - ref to error array for client feedback
 *
 * @ret   bool   -true if the directory/file is updated in the database
 *
 */

//new function to recursively change the database fpath entries where a directory has been renamed or moved
//only used where storage is to the database
// the first parameter is a file record for the source directory (which has already had it's fpath changed to the correct one)
// the second parameter allows the return condition to be passed on, the third is the address of an array where error messages
// can be stored
//NOTE there can be no permission checking here otherwise you might end up with some files re-pathed and others not
function dbfm_recur_dbrename($source, $ret, &$err) {
  $currentdate = date("Y-m-d H:i:s");
  //look in the database for children of the directory
  //for each child, re-path it. If it's a directory recurse and re-path all its children
  $qkidlist = db_query("SELECT fid, fpath, fname, fsize, fmime FROM {dbfm_file} WHERE fparent = %d", $source->fid);
  while ($sdata = db_fetch_object($qkidlist)) {
    //update the object
    $new_path = $source->fpath . "/" . $sdata->fname;

    $result = db_query("UPDATE {dbfm_file} SET fpath = '%s', flastmod = '%s' WHERE fid = %d", $new_path, $currentdate, $sdata->fid);
    if ($result === FALSE) {
      $err = "ERROR: Rename to " . $new_path . " for " . $sdata->fname . " failed";
      $ret = FALSE;
    }

    // if it's a directory or version directory recurse (assuming there are entries in it - versions always have at least one entry)
    if (($sdata->fmime == 'directory' AND $sdata->fsize > 0) || ($sdata->fmime == 'version')) {
      // update the source record first 
      $sdata->fpath = $new_path;
      $ret = dbfm_recur_dbrename($sdata, $ret, $err);
    }

  } // end of while loop
  return $ret;
}


/**
 * dbfm_delete - called from the ajax action - switch case 'delete':
 *
 * @param string $source - the source directory path
 * @param string &$err - ref to error array for client feedback
 *
 * @ret   bool   -true if the directory or file is deleted and all file records updated
 */
 //initally I thought that just setting the top of the tree to -1 
 //would be a good solution - you can't actually
 //get to any of the files on the tree. BUT when you're adding a new file you need to check whether that file already exists
 //in that case you'd need to search the whole tree to see whether there's a -1 anywhere - UGHH. That slows every download
 //so the real solution is to add some recursive code to mark every file in every subdirectory with a parent of minus
 //the current parent value, in that way it would stil be possible (although not yet implemented ) to undelete deleted files
 //and directories.
 //ok - that's done BUT we still can't have an administritive view showing the deleted stuff because the fids are still all positive
 //perhaps an easier solution is just to mark each record as deleted in a new field????
function dbfm_delete($rec_to_delete, &$err) {
  $currentdate = date("Y-m-d H:i:s");

  // in fact there's a sneaky way of dealing with the whole thing... we'll just mark the parent record id as -(fid)
  // fill in the last modified date. then have an admin function that REALLY deletes these entries at a later date
  // this means we don't need a recursive directory delete (at least not here, it will be in admin). 
  // Directories and their contents become invisible
  
  //extra check added - if we're deleting a file which has a version directory as its parent you may NOT delete the last revision
  //open the parent record
  if (($par_rec = dbfm_selected_file_record($rec_to_delete->fparent, '', $err)) == FALSE) {
    $err = "ERROR: Delete Failed - Unable to open the parent directory - (fid = " . $rec_to_delete->fparent . ")";
    return FALSE;
  }
  
  //if it's a version, see how many children it has
  if ($par_rec->fmime == 'version' AND $par_rec->fsize == 1) {
    $err = "Delete Failed - You may not delete the last revision in a versions directory";
    return FALSE;
  }

  //if the parent record is a version and you're trying to delete the 'current' version you will be asked to de-select it
  if ($par_rec->fmime == 'version' AND $par_rec->fchild == $rec_to_delete->fid) {
    $err = "Delete Failed - You may not delete the 'current' revision";
    return FALSE;
  }

  //if it's a single file just update the file with a negative parent id (also change the modified date). Otherwise, if it's a directory
  //we first edit the record then we call a recursive function that changes all the files and subdirectories underneath
  $result = db_query("UPDATE {dbfm_file} SET fparent = %d, flastmod = '%s' WHERE fid = %d", -($rec_to_delete->fparent), $currentdate, $rec_to_delete->fid);
  if ($result === FALSE) {
    $err = "ERROR: Delete Failed for " . $rec_to_delete->fname . ", unable to update record";
    return FALSE;
  }

  //need to update the parent record to reduce the filecount
  $newfilecount = $par_rec->fsize - 1;
  $result = db_query("UPDATE {dbfm_file} SET fsize = %d, flastmod = '%s' WHERE fid = %d", $newfilecount, $currentdate, $par_rec->fid);
  if ($result === FALSE) {
    $err = "ERROR: Unable to update filecount for directory " . $par_rec->fname;
    return FALSE;
  }
  if (($rec_to_delete->fmime == 'directory' && $rec_to_delete->fsize > 0) || $rec_to_delete->fmime == 'version') {
    //call a recursive function to delete all the children - parameters are the record 
    //the return value and any error message
    $ret = TRUE;
    $err = "Delete successful";

    $fred = dbfm_delete_recur($rec_to_delete, $ret, $err);
    if (! $ret) { 
      return FALSE;
    }
  }
  return TRUE;
}

/**
 * Delete a directory and all its files and subdirectories
 *
 * @param string  $source - the source directory path (what you're deleting)
 * @param bool  &$ret - ref to the return value (True or False) - this is passed as an address by the calling program
 * @param string  &$err - ref to error array for client feedback - this is passed as an address by the calling program
 *
 */
function dbfm_delete_recur($source, &$ret, &$err) {
  //look in the database for children of the directory
  //for each child, reset the modified date and parent. If it's a directory recurse and modify all its children
  $currentdate = date("Y-m-d H:i:s");
  $qkidlist = db_query("SELECT fid, fparent, fmime, fname, fsize FROM {dbfm_file} WHERE fparent = %d", $source->fid);
  while ($sdata = db_fetch_object($qkidlist)) {
    //update the object
    $result = db_query("UPDATE {dbfm_file} SET fparent = '%d', flastmod = '%s' WHERE fid = %d", -($sdata->fparent), $currentdate, $sdata->fid);
    if ($result === FALSE) {
      $err = "ERROR: Recursive delete has failed for " . $sdata->fname;
      $ret = FALSE;
    }

    // if it's a directory, has it got any entries, if so recurse
    if (($sdata->fmime == 'directory' && $sdata->fsize > 0) || $sdata->fmime == 'version') {
      $ret = dbfm_delete_recur($sdata, $ret, $err);
    }

  } // end of while loop
  return $ret;
}

/**
 * Move a directory or file and update database
 *
 * @param string  $source - the source directory path (what you're moving)
 * @param string  $dest - the destination directory name (where you're moving it)
 * @param string  &$err - ref to an error string for client feedback
 *
 * @ret   bool   -true if the file and the database entries are updated successfully
 *
 */
function dbfm_move($source_rec, $dest_rec, &$err) {

  $currentdate = date("Y-m-d H:i:s");

  //we'll need this to decrement the filecount once the file or directory has been moved
  if(($parent_rec = dbfm_selected_file_record($source_rec->fparent, '', $err)) == FALSE) {
    $err = "Move failed - unable to open parent directory for " . $source_rec->fpath;
    return FALSE;
  }

  // check the destination is a directory and not a file, GUI should prevent this
  if ($dest_rec->fmime != 'directory') {
    $err = "Move failed - destination " . $dest_rec->fname . " must be a directory";
    return FALSE;
  }

  $new_path = $dest_rec->fpath . '/' . $source_rec->fname;

  // we first check whether a file / directory of that name already exists in the destination 
  // directory. (that's if the directory has any entries)
  if ($dest_rec->fsize > 0) {
    //what happens here is they actually WANT to put a file of the same name in?? They might want to shove stuff into
    //a 'version'?? For now we just can't allow it.
    $qdup = db_query("SELECT fid FROM {dbfm_file} WHERE fpath = '%s' AND fparent > 0", $new_path);
    $numret = db_num_rows($qdup);
    //look through the children in the destination directory - if one has the same name then
    if ($numret > 0) {
      $err = "An object of that name already exists in the destination directory";
      return FALSE;
    }
  }
  
  //edit the curent record to correct the parent and change the path
  $result = db_query("UPDATE {dbfm_file} SET fpath = '%s', fparent = %d, flastmod = '%s' WHERE fid = %d", $new_path, $dest_rec->fid, $currentdate, $source_rec->fid);
  if ($result === FALSE) {
    $err = "ERROR: Unable to update the database path for move to " . $new_path;
    return FALSE;
  }

  //edit the destination parent record to add one to the number of entries held in the fsize field
  $result = db_query("UPDATE {dbfm_file} SET fsize = %d, flastmod = '%s' WHERE fid = %d", $dest_rec->fsize + 1, $currentdate, $dest_rec->fid);
  if ($result === FALSE) {
    $err = "ERROR: Unable to update the destination parent directory " . $dest_rec->fpath;
    return FALSE;
  }

  //edit the previous parent record to deduct one from the number of entries held in the fsize field
  $result = db_query("UPDATE {dbfm_file} SET fsize = %d, flastmod = '%s' WHERE fid = %d", $parent_rec->fsize - 1, $currentdate, $parent_rec->fid);
  if ($result === FALSE) {
    $err = "ERROR: Unable to update the previous parent directory " . $parentpath;
    return FALSE;
  }

  $ret = TRUE;
  //test whether the source is a directory and has files in it, if so we need to recursively change all the children of the directory
  //similarly versions need to have their children (there will always be at least one) dealt with
  if (($source_rec->fmime == 'directory' && $source_rec->fsize > 0) || $source_rec->fmime == 'version') {
    //we need to update the source record first 
    $source_rec->fpath = $new_path;
    $ret = dbfm_recur_dbrename($source_rec, $ret, $err);
  }
  return $ret;
}

?>
